﻿using System;
using System.IO;
using System.Reflection;

namespace ConsoleApplication1
{
    class Program
    {
        static void Main(string[] args)
        {
            string pluginFolder = @".\plugin";
            if (Directory.Exists(pluginFolder) == true)
            {
                ProcessPlugIn(pluginFolder);
            }
        }

        private static void ProcessPlugIn(string rootPath)
        {
            foreach (string dllPath in Directory.EnumerateFiles(rootPath, "*.dll"))
            {
                Assembly pluginDll = Assembly.LoadFrom(dllPath);

                Type entryType = FindEntryType(pluginDll);
                if (entryType == null)
                {
                    continue;
                }

                object instance = Activator.CreateInstance(entryType);

                MethodInfo entryMethod = CallStartupMethod(entryType);
                if (entryMethod == null)
                {
                    continue;
                }

                entryMethod.Invoke(instance, null);
            }
        }

        private static MethodInfo CallStartupMethod(Type entryType)
        {
            foreach (MethodInfo methodInfo in entryType.GetMethods())
            {
                foreach (object objAttribute in methodInfo.GetCustomAttributes(false))
                {
                    if (objAttribute.GetType().Name == "StartupAttribute")
                    {
                        return methodInfo;
                    }
                }
            }

            return null;
        }

        private static Type FindEntryType(Assembly pluginDll)
        {
            foreach (Type type in pluginDll.GetTypes())
            {
                foreach (object objAttr in type.GetCustomAttributes(false))
                {
                    if (objAttr.GetType().Name == "PluginAttribute")
                    {
                        return type;
                    }
                }
            }

            return null;
        }
    }
}
